@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -69,7 +69,7 @@ extern int burn_drive_role_4_allowed;
 */
 int burn_setup_drive(struct burn_drive *d, char *fname)
 {
-	d->devname = burn_strdup(fname);
+	d->devname = strdup(fname);
 	memset(&d->params, 0, sizeof(struct params));
 	d->idata = NULL;
 	d->mdata = NULL;
@@ -187,7 +187,10 @@ int burn_drive_is_occupied(struct burn_drive *d)
 	if(d->busy == BURN_DRIVE_READING_SYNC ||
 		 d->busy == BURN_DRIVE_WRITING_SYNC)
 		return 2;
-	if(d->busy == BURN_DRIVE_WRITING) {
+	if(d->busy == BURN_DRIVE_WRITING ||
+	   d->busy == BURN_DRIVE_WRITING_LEADIN ||
+	   d->busy == BURN_DRIVE_WRITING_LEADOUT ||
+	   d->busy == BURN_DRIVE_WRITING_PREGAP) {
 
 		/* ts A70928 */
 		/* >>> how do i learn whether the writer thread is still
@@ -340,7 +343,7 @@ int burn_drive_send_default_page_05(struct burn_drive *d, int flag)
 	else
 		burn_write_opts_set_write_type(opts,
 			BURN_WRITE_SAO, BURN_BLOCK_SAO);
-	d->send_write_parameters(d, opts);
+	d->send_write_parameters(d, NULL, -1, opts);
 	burn_write_opts_free(opts);
 	d->sent_default_page_05 = 1;
 	return 1;
@@ -483,9 +486,10 @@ int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 int burn_drive_grab(struct burn_drive *d, int le)
 {
 	int errcode;
-	/* ts A61125 - B10314 */
-	int ret, sose;
+	/* ts A61125 - B20122 */
+	int ret, sose, signal_action_mem = -1;
 
+	sose = d->silent_on_scsi_error;
 	if (!d->released) {
                 libdax_msgs_submit(libdax_messenger, d->global_index,
                                 0x00020189, LIBDAX_MSGS_SEV_FATAL,
@@ -502,26 +506,40 @@ int burn_drive_grab(struct burn_drive *d, int le)
 	errcode = d->grab(d);
 	if (errcode == 0)
 		return 0;
+
+	burn_grab_prepare_sig_action(&signal_action_mem, 0);
 	d->busy = BURN_DRIVE_GRABBING;
 
 	if (le)
 		d->load(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	d->lock(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	/* ts A61118 */
 	d->start_unit(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	/* ts A61202 : gave bit1 of le a meaning */
-	sose = d->silent_on_scsi_error;
 	if (!le)
 		d->silent_on_scsi_error = 1;
 	/* ts A61125 : outsourced media state inquiry aspects */
 	ret = burn_drive_inquire_media(d);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
 
 	burn_drive_send_default_page_05(d, 0);
+	if (d->cancel || burn_is_aborting(0))
+		{ret = 0; goto ex;}
+
+ex:;
 	d->silent_on_scsi_error = sose;
 	d->busy = BURN_DRIVE_IDLE;
+	burn_grab_restore_sig_action(signal_action_mem, 0);
 	return ret;
 }
 
@@ -745,7 +763,7 @@ void burn_drive_release(struct burn_drive *d, int le)
 /* API */
 int burn_drive_re_assess(struct burn_drive *d, int flag)
 {
-	int ret;
+	int ret, signal_action_mem;
 
 	if (d->released) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
@@ -762,10 +780,12 @@ int burn_drive_re_assess(struct burn_drive *d, int flag)
 		return ret;
 	}
 
+	burn_grab_prepare_sig_action(&signal_action_mem, 0);
 	d->busy = BURN_DRIVE_GRABBING;
 	ret = burn_drive_inquire_media(d);
 	burn_drive_send_default_page_05(d, 0);
 	d->busy = BURN_DRIVE_IDLE;
+	burn_grab_restore_sig_action(signal_action_mem, 0);
 	d->released = 0;
 	return ret;
 }
@@ -863,12 +883,34 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 	d->erase(d, fast);
 	d->busy = BURN_DRIVE_ERASING;
 
+#ifdef Libburn_old_progress_looP
+
 	/* read the initial 0 stage */
 	while (!d->test_unit_ready(d) && d->get_erase_progress(d) == 0)
 		sleep(1);
 	while ((d->progress.sector = d->get_erase_progress(d)) > 0 ||
 		!d->test_unit_ready(d))
 		sleep(1);
+
+#else /* Libburn_old_progress_looP */
+
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if (ret == -2 || ret > 0)
+	break;
+		sleep(1);
+	}
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if(ret == -2)
+	break;
+		if (ret >= 0)
+			d->progress.sector = ret;
+		sleep(1);
+        }
+
+#endif /* ! Libburn_old_progress_looP */
+
 	d->progress.sector = 0x10000;
 
 	/* ts A61125 : update media state records */
@@ -913,6 +955,8 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 	if (ret <= 0)
 		d->cancel = 1;
 
+#ifdef Libburn_old_progress_looP
+
 	while (!d->test_unit_ready(d) && d->get_erase_progress(d) == 0)
 		sleep(1);
 	while ((pseudo_sector = d->get_erase_progress(d)) > 0 ||
@@ -920,6 +964,26 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 		d->progress.sector = pseudo_sector / stages;
 		sleep(1);
         }
+
+#else /* Libburn_old_progress_looP */
+
+	while (1) {
+		ret = d->get_erase_progress(d);
+		if (ret == -2 || ret > 0)
+	break;
+		sleep(1);
+	}
+	while (1) {
+		pseudo_sector = d->get_erase_progress(d);
+		if(pseudo_sector == -2)
+	break;
+		if (pseudo_sector >= 0)
+			d->progress.sector = pseudo_sector / stages;
+		sleep(1);
+        }
+
+#endif /* ! Libburn_old_progress_looP */
+
 	d->sync_cache(d);
 
 	if (size <= 0)
@@ -2318,12 +2382,14 @@ int burn_abort_5(int patience,
 			}
 
 			if(occup < 10) {
+				if (!drive_array[i].cancel)
+					burn_drive_cancel(&(drive_array[i]));
 				if (drive_array[i].drive_role != 1)
 		 			/* occup == -1 comes early */
 					usleep(1000000);
 				burn_drive_forget(&(drive_array[i]), 1);
 			} else if(occup <= 100) {
-				if(first_round)
+				if (!drive_array[i].cancel)
 					burn_drive_cancel(&(drive_array[i]));
 				still_not_done++;
 			} else if(occup <= 1000) {
@@ -2469,7 +2535,7 @@ int burn_disc_track_lba_nwa(struct burn_drive *d, struct burn_write_opts *o,
 	if (d->drive_role != 1)
 		return 0;
 	if (o != NULL)
-		d->send_write_parameters(d, o);
+		d->send_write_parameters(d, NULL, -1, o);
 	ret = d->get_nwa(d, trackno, lba, nwa);
 	return ret;
 }
@@ -2522,7 +2588,7 @@ off_t burn_disc_available_space(struct burn_drive *d,
 			  (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
 	} else {
 		if (o != NULL)
-			d->send_write_parameters(d, o);
+			d->send_write_parameters(d, NULL, -1, o);
 		d->get_nwa(d, -1, &lba, &nwa);
 	}
 	if (o != NULL) {
@@ -2971,11 +3037,12 @@ int burn_disc_get_write_mode_demands(struct burn_disc *disc,
 				last_track_is_unknown = 1;
 			} else
 				last_track_is_unknown = 0;
-			if (mode != track->mode)
+			if ((mode & BURN_MODE_BITS) !=
+						(track->mode & BURN_MODE_BITS))
 				result->mixed_mode = 1;
-			if (track->mode == BURN_MODE1) {
+			if (track->mode & BURN_MODE1) {
 				result->block_types |= BURN_BLOCK_MODE1;
-			} else if (track->mode == BURN_AUDIO) {
+			} else if (track->mode & BURN_AUDIO) {
 				result->audio = 1;
 				result->block_types |= BURN_BLOCK_RAW0;
 				result->exotic_track = 1;
@@ -3309,3 +3376,16 @@ int burn_disc_next_track_is_damaged(struct burn_drive *d, int flag)
 }
 
 
+/* ts B11201 : API */
+/* Read the CD-TEXT data from the Lead-in of an Audio CD
+*/
+int burn_disc_get_leadin_text(struct burn_drive *d,
+                              unsigned char **text_packs, int *num_packs,
+                              int flag)
+{
+	int ret;
+
+	ret = mmc_get_leadin_text(d, text_packs, num_packs, 0);
+	return ret;
+}
+
