@@ -1,5 +1,11 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
+
 #ifndef __TRANSPORT
 #define __TRANSPORT
 
@@ -40,7 +46,9 @@ struct buffer
 	   Added 4096 bytes reserve against possible buffer overflows.
 	   (Changed in sector.c buffer flush test from >= to > BUFFER_SIZE .
 	    This can at most cause a 1 sector overlap. Sometimes an offset
-	    of 16 byte is applied to the output data (in some RAW mode). ) */
+	    of 16 byte is applied to the output data (in some RAW mode). )
+	    burn_write_opts.cdxa_conversion can imply an offset of 8 bytes.
+	 */
 	unsigned char data[BUFFER_SIZE + 4096];
 	int sectors;
 	int bytes;
@@ -56,6 +64,7 @@ struct command
 	int error;
 	int retry;
 	struct buffer *page;
+	int timeout; /* milliseconds */
 };
 
 struct burn_scsi_inquiry_data
@@ -126,6 +135,8 @@ struct burn_drive
 		1=MMC drive ,
 		2=stdio random read-write
 		3=stdio sequential write-only
+                4=stdio random read-only
+                5=stdio random write-only
 	*/
 	int drive_role;
 
@@ -161,6 +172,11 @@ struct burn_drive
 	char current_profile_text[80];
 	int current_is_cd_profile;
 	int current_is_supported_profile;
+	/* ts A90603 */
+	int current_is_guessed_profile;
+	/* ts A90815 */
+	unsigned char all_profiles[256];
+	int num_profiles;
 
 	/* ts A70128 : MMC-to-MMC feature info from 46h for DVD-RW.
            Quite internal. Regard as opaque :)
@@ -193,6 +209,11 @@ struct burn_drive
 	*/
 	int current_feat2fh_byte4;
 
+	/* ts B10524 : whether the damage bit was set for the future track.
+	               bit0= damage bit , bit1= nwa valid bit
+	*/
+	int next_track_damaged;
+
 	/* ts A70114 : whether a DVD-RW media holds an incomplete session
 	               (which could need closing after write) */
 	int needs_close_session;
@@ -211,7 +232,19 @@ struct burn_drive
         int stream_recording_start;
 
 	/* ts A61218 from 51h READ DISC INFORMATION */
+	int last_lead_in;
+	int last_lead_out;
+	int num_opc_tables;   /* ts A91104: -1= not yet known */
 	int bg_format_status; /* 0=needs format start, 1=needs format restart*/
+	int disc_type; /* 0="CD-DA or CD-ROM", 0x10="CD-I", 0x20="CD-ROM XA" */
+	unsigned int disc_id; /* a "32 bit binary integer" */
+	char disc_bar_code[9];
+	int disc_app_code;
+	int disc_info_valid;  /* bit0= disc_type ,     bit1= disc_id ,
+				 bit2= disc_bar_code , bit3= disc_app_code
+				 bit4= URU bit is set (= unrestricted use)
+				 bit5= Erasable bit was set in reply
+			       */
 
 	/* ts A70108 from 23h READ FORMAT CAPACITY mmc5r03c.pdf 6.24 */
 	int format_descr_type;      /* 1=unformatted, 2=formatted, 3=unclear */
@@ -228,8 +261,18 @@ struct burn_drive
 	volatile int released;
 
 	/* ts A61106 */
+	/* 0= report errors
+	   1= do not report errors
+	   2= do not report errors which the libburn function indicates in
+	      member .had_particular_error
+	*/
 	int silent_on_scsi_error;
 
+	/* ts B21023 */
+	/* bit0= 5 64 00 occured with READ10 in mmc_read_10()
+	*/
+	int had_particular_error;
+
 	int stdio_fd;
 
 	int nwa;		/* next writeable address */
@@ -247,6 +290,10 @@ struct burn_drive
 	/* ts A70129 :
 	   from 51h READ DISC INFORMATION Last Track Number in Last Session */
 	int last_track_no;
+
+	/* ts B10730 : whether a default mode page 05 was already sent.
+	*/
+	int sent_default_page_05;
 	/* ts A70212 : from various sources : free space on media (in bytes)
 	               With CD this might change after particular write
 	               parameters have been set and nwa has been inquired.
@@ -257,15 +304,30 @@ struct burn_drive
 	int media_lba_limit;
 
 	/* ts A81210 : Upper limit of readable data size,
-	               0x7fffffff = unknown */
+	               0x7fffffff = unknown
+	               0x7ffffff0 = 32 bit overflow, or unknown stdio size
+	 */
 	int media_read_capacity;
 
+	/* ts B10314 : Next Writeable Adress for drive_role == 5 */
+        int role_5_nwa;
+
 	int toc_temp;
 	struct burn_disc *disc;	/* disc structure */
 	int block_types[4];
 	struct buffer *buffer;
 	struct burn_progress progress;
 
+	/* To be used by mmc.c, sbc.c, spc.c for SCSI commands where the struct
+	   content surely does not have to persist while another command gets
+	   composed and executed.
+	   (Inherently, sending SCSI commands to the same drive cannot be
+	    thread-safe. But there are functions which send SCSI commands
+	    and also call other such functions. These shall use own allocated
+	    command structs and not this struct here.)
+	*/
+	struct command casual_command;
+
 	/* ts A70711 : keeping an eye on the drive buffer */
 	off_t pessimistic_buffer_free;
 	int pbf_altered;
@@ -287,6 +349,8 @@ struct burn_drive
 	/* ts A70929 */
 	pid_t thread_pid;
 	int thread_pid_valid;
+	/* ts B00225 */
+	pthread_t thread_tid;
 
 
 /* transport functions */
@@ -312,18 +376,24 @@ struct burn_drive
 	void (*eject) (struct burn_drive *);
 	void (*load) (struct burn_drive *);
 	int (*start_unit) (struct burn_drive *);
+
+	/* ts A90824 : Calming down noisy drives */
+	int (*stop_unit) (struct burn_drive *);
+	int is_stopped;
+
 	void (*read_disc_info) (struct burn_drive *);
-	void (*read_sectors) (struct burn_drive *,
-			      int start,
-			      int len,
-			      const struct burn_read_opts *, struct buffer *);
+	int (*read_cd) (struct burn_drive *, int start, int len,
+	                int sec_type, int main_ch,
+	                const struct burn_read_opts *, struct buffer *,
+	                int flag);
 	void (*perform_opc) (struct burn_drive *);
 	void (*set_speed) (struct burn_drive *, int, int);
 	void (*send_parameters) (struct burn_drive *,
 				 const struct burn_read_opts *);
 	void (*send_write_parameters) (struct burn_drive *,
+				       struct burn_session *, int tno,
 				       const struct burn_write_opts *);
-	void (*send_cue_sheet) (struct burn_drive *, struct cue_sheet *);
+	int (*send_cue_sheet) (struct burn_drive *, struct cue_sheet *);
 
 	/* ts A70205 : Announce size of a DVD-R[W] DAO session. */
 	int (*reserve_track) (struct burn_drive *d, off_t size);
